#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Following a PSD tutorial found here: https://medialoot.com/blog/how-to-make-a-opart-text-effect-in-photoshop/

# author: Pocholo
# date: 12/6/21

# Comments directed to http://gimpchat.com

# Copyright (C)

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by   
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.


from gimpfu import *
import math
import gettext
import os, sys,time 

gettext.install("gimp20", gimp.locale_directory, unicode=True)

def pm_create_an_opart_text_effect(customtext, font, size, color):
          
    #Initiates the defaults
    pdb.gimp_context_set_defaults()
	    
    #Create the image
    img = pdb.gimp_image_new(1, 1, RGB)
    
    #Create the displace text 	
    pdb.gimp_context_set_foreground("Black")
    textLayer = pdb.gimp_text_fontname(img, None, 0, 0, customtext, 20,
                                   True, size, PIXELS, font)
    pdb.gimp_text_layer_set_justification(textLayer, 2)
    pdb.gimp_text_layer_set_letter_spacing(textLayer, 50)
    img.resize(textLayer.width, textLayer.height, 0, 0)
    dispLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Displace text", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, dispLayer, 1)
    pdb.gimp_context_set_foreground("White")
    pdb.gimp_drawable_fill(dispLayer, FILL_FOREGROUND)
    dispLayer = pdb.gimp_image_merge_down(img, textLayer, CLIP_TO_IMAGE)
    pdb.plug_in_gauss(img, dispLayer, 50, 50, 1)   #method=1
    pdb.gimp_layer_set_mode(dispLayer, LAYER_MODE_MULTIPLY)
    
    #Process
    pdb.gimp_display_new(img)
    
    #Create the line background
    backLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Background", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, backLayer, 2)
    pdb.gimp_context_set_foreground((128, 128, 128))
    pdb.gimp_drawable_fill(backLayer, FILL_FOREGROUND)
    pdb.plug_in_newsprint(img, backLayer, 
                                      10,   #cell_width
                                       2,   #colorspace - CMYK
                                       0,   #k_pullout
                                      45,   #gry_ang
                                       1,   #gry_spotfn
                                      45,   #red_ang
                                       1,   #red_spotfn
                                      45,   #grn_ang
                                       1,   #grn_spotfn
                                      45,   #blu_ang
                                       1,   #blu_spotfn
                                      16)   #oversample
                                      
    #Line color
    colorLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Layer", 100, LAYER_MODE_MULTIPLY)
    pdb.gimp_image_add_layer(img, colorLayer, 1)
    pdb.gimp_context_set_foreground(color)
    pdb.gimp_drawable_fill(colorLayer, FILL_FOREGROUND) 
    
    #Create the effect
    pdb.plug_in_displace(img, backLayer, 
                                     60,   #amount_x
                                     30,   #amount_y
                                   TRUE,   #do_x
                                   TRUE,   #do_y
                              dispLayer,   #displace_map_x
                              dispLayer,   #displace_map_y
                                      1)   #displace_type - wrap   
    backLayer = pdb.gimp_image_merge_visible_layers(img, CLIP_TO_BOTTOM_LAYER)
    
    #Set the defaults
    pdb.gimp_context_set_defaults()
       
register(
    "pm_create_an_opart_text_effect",
    "Creates a retro optical text effect",
    "Creates a retro optical text effect",
    "Pocholo",
    "Pocholo",
    "2021",
    "Create an OpArt Text Effect",
    "",
    [
    (PF_STRING, "customtext", "Text string", "GIMP"),
    (PF_FONT, "font", "Font face", "Cooper Black,"),
    (PF_SPINNER, "size", "Text size", 150, (100, 200, 1)),
    (PF_COLOR, "color", "OpArt Color:", (155, 0, 255)),
    ],
    [],
    pm_create_an_opart_text_effect, menu="<Image>/Pocholo-scripts/Create an OpArt Test",
    domain=("gimp20", gimp.locale_directory))
main()								